<?php
/**
 * Class file that is wrapper for attendee sub-event information
 *
 * @since 3.8.8
 * @package rsvp-pro-plugin
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) :
	exit;
endif;

/**
 * Class for handling attendee sub-event information and storing it into the database.
 *
 * @since 3.8.8
 */
class Rsvp_Pro_Attendee_Subevent {

	/**
	 * The database identifier for the sub-event record.
	 *
	 * @var integer
	 */
	private $id = 0;

	/**
	 * The RSVP date for this entry.
	 *
	 * @var string
	 */
	private $rsvp_date = '';

	/**
	 * The RSVP status for this sub-event.
	 *
	 * @var string
	 */
	private $rsvp_status = 'NoResponse';

	/**
	 * The event database identifier associated with this record.
	 *
	 * @var integer
	 */
	private $event_id = 0;

	/**
	 * The attendee ID associated with this record.
	 *
	 * @var integer
	 */
	private $attendee_id = 0;

	/**
	 * If the attendee has checked in to the event or not.
	 *
	 * @var string
	 */
	private $checked_in = 'N';

	/**
	 * The attendee unique token.
	 *
	 * @var string
	 */
	private $token = '';

	/**
	 * Holds all of the answers for custom questions that this attendee has added.
	 *
	 * @var array
	 */
	private $custom_question_answers = array();

	/**
	 * Constructor for this record.
	 *
	 * @param integer $id The database identifier that we want to load the information from.
	 */
	public function __construct( $id = 0 ) {
		$this->id = $id;
		$this->load();
	}

	/**
	 * Loads the information for this sub-event database record.
	 */
	private function load() {
		global $wpdb;

		if ( 0 < $this->id ) {
			$sql      = 'SELECT rsvpDate, rsvpStatus, rsvpEventID, rsvpAttendeeID, 
				checked_in, token FROM ' . PRO_ATTENDEE_SUB_EVENTS_TABLE . ' WHERE id = %d';
			$attendee = $wpdb->get_row( $wpdb->prepare( $sql, $this->id ) );

			if ( false !== $attendee ) {
				$this->rsvp_date   = date_i18n( get_option( 'date_format' ), strtotime( $attendee->rsvpDate ) );
				$this->rsvp_status = rsvp_pro_format_rsvp_status_for_database( $attendee->rsvpStatus );
				$this->event_id    = $attendee->rsvpEventID;
				$this->attendee_id = $attendee->rsvpAttendeeID;
				$this->checked_in  = $attendee->checked_in;
				$this->token       = stripslashes_deep( $attendee->token );
			}

			$this->custom_question_answers = Rsvp_Pro_Attendee_Answers::load_answers_for_attendee( $this->id, $this->event_id );
		}
	}

	/**
	 * Saves the sub-event information to the database.
	 */
	public function save() {
		global $wpdb;

		if ( 0 < $this->id ) {
			$wpdb->update(
				PRO_ATTENDEE_SUB_EVENTS_TABLE,
				array(
					'rsvpDate'       => date( 'Y-m-d', strtotime( $this->rsvp_date ) ),
					'rsvpStatus'     => rsvp_pro_format_rsvp_status_for_database( $this->rsvp_status ),
					'rsvpEventID'    => $this->event_id,
					'rsvpAttendeeID' => $this->attendee_id,
					'checked_in'     => $this->checked_in,
				),
				array( 'id' => $this->id ),
				array(
					'%s',
					'%s',
					'%d',
					'%d',
					'%s',
				),
				array( '%d' )
			);
		} else {
			$wpdb->insert(
				PRO_ATTENDEE_SUB_EVENTS_TABLE,
				array(
					'rsvpDate'       => date( 'Y-m-d', strtotime( $this->rsvp_date ) ),
					'rsvpStatus'     => rsvp_pro_format_rsvp_status_for_database( $this->rsvp_status ),
					'rsvpEventID'    => $this->event_id,
					'rsvpAttendeeID' => $this->attendee_id,
					'checked_in'     => $this->checked_in,
					'token'          => rsvp_pro_generate_attendee_token(),
				),
				array(
					'%s',
					'%s',
					'%d',
					'%d',
					'%s',
					'%s',
				)
			);
		}
	}

	/**
	 * Deletes the sub-event information to the database.
	 */
	public function delete() {
		global $wpdb;

		if ( 0 < $this->id ) {
			$wpdb->delete(
				PRO_ATTENDEE_SUB_EVENTS_TABLE,
				array( 'id' => $this->id ),
				array( '%d' )
			);
		}
	}

	/**
	 * Gets the sub-event information for a specific attendee.
	 *
	 * @param  int $attendee_id The attendee ID that we want to get the sub-event information for.
	 *
	 * @return array              Array of Rsvp_Pro_Attendee_Subevent for each sub-event.
	 */
	public static function load_subevents_for_attendee( $attendee_id ) {
		global $wpdb;

		$subevents     = array();
		$sql           = 'SELECT id FROM ' . PRO_ATTENDEE_SUB_EVENTS_TABLE . ' WHERE rsvpAttendeeID = %d';
		$subevent_info = $wpdb->get_results( $wpdb->prepare( $sql, $attendee_id ) );
		foreach ( $subevent_info as $se ) {
			$subevents[] = new Rsvp_Pro_Attendee_Subevent( $se->id );
		}

		return $subevents;
	}

	/**
	 * Gets the database identifier for this sub-event record.
	 *
	 * @return int The database identifier for this record.
	 */
	public function get_id() {
		return $this->id;
	}

	/**
	 * Gets the unique token for the sub-event record.
	 *
	 * @return string The token for this sub-event record.
	 */
	public function get_token() {
		return $this->token;
	}

	/**
	 * Returns the RSVP date for this sub-event.
	 *
	 * @return date The RSVP date formatted in the current WordPress instance date format.
	 */
	public function get_rsvp_date() {
		return $this->rsvp_date;
	}

	/**
	 * Sets the RSVP date for this sub-event.
	 *
	 * @param string $rsvp_date Holding a date in string format.
	 */
	public function set_rsvp_date( $rsvp_date ) {
		if ( false === strtotime( $rsvp_date ) ) {
			$rsvp_date = '';
		}

		$this->rsvp_date = $rsvp_date;
	}

	/**
	 * The RSVP status for this sub-event.
	 *
	 * @return string The RSVP status for this sub-event.
	 */
	public function get_rsvp_status() {
		return $this->rsvp_status;
	}

	/**
	 * Sets the RSVP status for this sub-event.
	 *
	 * @param string $status The RSVP status for this sub-event.
	 */
	public function set_rsvp_status( $status ) {
		$this->rsvp_status = rsvp_pro_format_rsvp_status_for_database( $status );
	}

	/**
	 * Returns the event ID for this sub-event.
	 *
	 * @return int The event ID related to this sub-event.
	 */
	public function get_event_id() {
		return $this->event_id;
	}

	/**
	 * Sets the event ID for this sub-event.
	 *
	 * @param int $event_id The event ID for this sub-event.
	 */
	public function set_event_id( $event_id ) {
		if ( ! is_numeric( $event_id ) || ( 0 >= $event_id ) ) {
			$event_id = 0;
		}

		$this->event_id = $event_id;
	}

	/**
	 * Gets the attendee ID for this sub-event.
	 *
	 * @return int The attendee ID for this sub-event.
	 */
	public function get_attendee_id() {
		return $this->attendee_id;
	}

	/**
	 * Sets the attendee ID for this sub-event.
	 *
	 * @param int $attendee_id The attendee ID for the sub-event.
	 */
	public function set_attendee_id( $attendee_id ) {
		if ( ! is_numeric( $attendee_id ) || ( 0 >= $attendee_id ) ) {
			$attendee_id = 0;
		}

		$this->attendee_id = $attendee_id;
	}

	/**
	 * The check-in field for the sub-event.
	 *
	 * @return string Either a 'Y' or 'N' if the people are checked-in.
	 */
	public function get_checked_in() {
		return $this->checked_in;
	}

	/**
	 * Sets the check-in field for the sub-event.
	 *
	 * @param string $checked_in Either 'Y' or 'N' for the checked-in value.
	 */
	public function set_checked_in( $checked_in ) {
		if ( 'Y' === strtoupper( $checked_in ) ) {
			$this->checked_in = 'Y';
		} else {
			$this->checked_in = 'N';
		}
	}

	/**
	 * Gets all of the custom question answers for an attendee.
	 *
	 * @return array An array of Rsvp_Pro_Attendee_Answers.
	 */
	public function get_custom_question_answers() {
		return $this->custom_question_answers;
	}

	/**
	 * Adds a custom question answer for an attendee.
	 *
	 * @param Rsvp_Pro_Attendee_Answers $custom_question_answer The custom question answer we want to save for an attendee.
	 */
	public function add_custom_question_answer( $custom_question_answer ) {
		if ( is_a( $custom_question_answer, 'Rsvp_Pro_Attendee_Answers' ) ) {
			for ( $i = 0; $i < count( $this->custom_question_answers ); $i ++ ) {
				if ( $this->custom_question_answers[ $i ]->get_id() === $custom_question_answer->get_id() ) {
					$this->custom_question_answers[ $i ] = $custom_question_answer;

					return;
				}
			}

			$this->custom_question_answers[] = $custom_question_answer;
		}
	}
}
